import sys
import time 
from PhidgetHelperFunctions import *
from Phidget22.Devices.DCMotor import *
from Phidget22.PhidgetException import *
from Phidget22.Phidget import *
from Phidget22.Net import *

'''
* Configures the device's DataInterval
* Displays info about the attached phidget channel.  
* Fired when a Phidget channel with onAttachHandler registered attaches
*
* @param self The Phidget channel that fired the attach event
'''
def onAttachHandler(self):
    
    ph = self

    try:
        """
        * Set the DataInterval inside of the attach handler to initialize the device with this value.
        * DataInterval defines the minimum time between VelocityUpdate events.
        * DataInterval can be set to any value from MinDataInterval to MaxDataInterval.
        """
        print("\tSetting DataInterval to 1000ms")
        try:
            ph.setDataInterval(1000)
        except PhidgetException as e:
            sys.stderr.write("Runtime Error -> Setting DataInterval: \n\t")
            DisplayError(e)
            return

        serialNumber = ph.getDeviceSerialNumber()
        channel = ph.getChannel()

        # Check if this is a VINT device
        try:
            hub = ph.getHub()
        except PhidgetException as e:
            if(e.code == ErrorCode.EPHIDGET_WRONGDEVICE): #Not a VINT Device
                print("\nAttach Event:\n\t-> Serial Number: " + str(serialNumber) +
                        "\n\t-> Channel " + str(channel) + "\n")
            return

        hubPort = ph.getHubPort()
       
        print("\nAttach Event:\n\t-> Serial Number: " + str(serialNumber) +
              "\n\t-> Hub Port: " + str(hubPort) + "\n\t-> Channel " + str(channel) + "\n")
        return
        
    except PhidgetException as e:
        print("\nError in Attach Event:")
        DisplayError(e)
        traceback.print_exc()
        return

"""
* Displays info about the detached phidget channel.
* Fired when a Phidget channel with onDetachHandler registered detaches
*
* @param self The Phidget channel that fired the attach event
"""
def onDetachHandler(self):

    ph = self

    try:
        serialNumber = ph.getDeviceSerialNumber()
    except PhidgetException as e:
        sys.stderr.write("Runtime Error -> Get DeviceSerialNumber: \n\t")
        DisplayError(e)
        return

    try:
        channel = ph.getChannel()
    except PhidgetException as e:
        sys.stderr.write("Runtime Error -> Get Channel: \n\t")
        DisplayError(e)
        return

    # Check if this is a VINT device
    try:
        hub = ph.getHub()
    except PhidgetException as e:
        if(e.code == ErrorCode.EPHIDGET_WRONGDEVICE):
            print("\nDetach Event:\n\t-> Serial Number: " + str(serialNumber) +
                    "\n\t-> Channel " + str(channel) + "\n")
        return
    try:
        hubPort = ph.getHubPort()
    except PhidgetException as e:
        sys.stderr.write("Runtime Error -> Get HubPort: \n\t")
        DisplayError(e)
        return

    print("\nDetach Event:\n\t-> Serial Number: " + str(serialNumber) +
          "\n\t-> Hub Port: " + str(hubPort) + "\n\t-> Channel " + str(channel) + "\n")
    return

"""
* Writes phidget error info to stderr.
* Fired when a Phidget channel with onErrorHandler registered encounters an error in the library
*
* @param self The Phidget channel that fired the attach event
* @param errorCode the code associated with the error of enum type ph.ErrorEventCode
* @param errorString string containing the description of the error fired
"""
def onErrorHandler(self, errorCode, errorString):

    sys.stderr.write("[Phidget Error Event] -> " + errorString + " (" + str(errorCode) + ")\n")

"""
* Outputs the DCMotor's most recently reported velocity.
* Fired when a DCMotor channel with onVelocityUpdateHandler registered meets DataInterval and ChangeTrigger criteria
*
* @param self The DCMotor channel that fired the VelocityUpdate event
* @param velocity The reported velocity from the DCMotor channel
"""
def onVelocityUpdateHandler(self, velocity):

    print("[Velocity Event] -> Velocity: " + str(velocity))

"""
* Prints descriptions of how events related to this class work
"""
def PrintEventDescriptions():

    print("\n--------------------\n"
        "\n  | Velocity update events contain the most recent velocity received from the device.\n"
        "  | The linked VelocityUpdate function will run as an event at every DataInterval.\n"
        "  | These events will not occur until a change in velocity >= to the set ChangeTrigger has occured.\n"
        "  | DataInterval and ChangeTrigger should initially be set in the device AttachHandler function."
        "  | Press ENTER once you have read this message.")
    readin = sys.stdin.readline(1)
    
    print("\n--------------------")
            
"""
* Creates, configures, and opens a DCMotor channel.
* Provides interface for controlling TargetVelocity of the DCMotor.
* Closes out DCMotor channel
*
* @return 0 if the program exits successfully, 1 if it exits with errors.
"""
def main():
    try:
        """
        * Allocate a new Phidget Channel object
        """
        try:
            ch = DCMotor()
        except PhidgetException as e:
            sys.stderr.write("Runtime Error -> Creating DCMotor: \n\t")
            DisplayError(e)
            raise
        except RuntimeError as e:
            sys.stderr.write("Runtime Error -> Creating DCMotor: \n\t" + e)
            raise

        """
        * Set matching parameters to specify which channel to open
        """
        #You may remove this line and hard-code the addressing parameters to fit your application
        channelInfo = AskForDeviceParameters(ch)
        
        ch.setDeviceSerialNumber(channelInfo.deviceSerialNumber)
        ch.setHubPort(channelInfo.hubPort)
        ch.setIsHubPortDevice(channelInfo.isHubPortDevice)
        ch.setChannel(channelInfo.channel)   
        
        if(channelInfo.netInfo.isRemote):
            ch.setIsRemote(channelInfo.netInfo.isRemote)
            if(channelInfo.netInfo.serverDiscovery):
                Net.enableServerDiscovery(PhidgetServerType.PHIDGETSERVER_DEVICEREMOTE)
            else:
                Net.addServer("Server", channelInfo.netInfo.hostname,
                    channelInfo.netInfo.port, channelInfo.netInfo.password, 0)
        
        """
        * Add event handlers before calling open so that no events are missed.
        """
        print("\n--------------------------------------")
        print("\nSetting OnAttachHandler...")
        ch.setOnAttachHandler(onAttachHandler)
        
        print("Setting OnDetachHandler...")
        ch.setOnDetachHandler(onDetachHandler)
        
        print("Setting OnErrorHandler...")
        ch.setOnErrorHandler(onErrorHandler)

        #This call may be harmlessly removed
        PrintEventDescriptions()
        
        print("\nSetting onVoltageUpdateHandler...")
        ch.setOnVelocityUpdateHandler(onVelocityUpdateHandler)
        
        """
        * Open the channel with a timeout
        """
        print("\nOpening and Waiting for Attachment...")
        
        try:
            ch.openWaitForAttachment(5000)
        except PhidgetException as e:
            PrintOpenErrorMessage(e, ch)
            raise EndProgramSignal("Program Terminated: Open Failed")

        print("--------------------\n"
        "\n  | DC motor speed can be controlled by setting its Target Velocity (ie its duty cycle).\n"
        "  | The target velocity can be a number from -1.0 to +1.0, where sign indicates direction of rotation.\n"
        "  | For this example, acceleration has been fixed to 1.0Hz, but can be changed in custom code.\n"

        "\nInput a desired velocity between -1.0 and 1.0 and press ENTER\n"
        "Input Q and press ENTER to quit\n");

        end = False

        while (end != True):
            buf = sys.stdin.readline(100)
            if not buf:
                continue

            if (buf[0] == 'Q' or buf[0] ==  'q'):
                end = True;
                continue
            
            try:
                velocity = float(buf)
            except ValueError as e:
                print("Input must be a number, or Q to quit.");
                continue

            if (velocity > 1.0):
                velocity = 1.0;
                print("MAXIMUM velocity is +/-1.0");

            if (velocity < -1.0):
                velocity = -1.0;
                print("MAXIMUM velocity is +/-1.0");

            print("Setting DCMotor TargetVelocity to " + str(velocity));
            ch.setTargetVelocity(velocity)

        '''
        * Perform clean up and exit
        '''

        #clear the VelocityUpdate event handler 
        ch.setOnVelocityUpdateHandler(None)  

        print("Cleaning up...")
        ch.close()
        print("\nExiting...")
        return 0

    except PhidgetException as e:
        sys.stderr.write("\nExiting with error(s)...")
        DisplayError(e)
        traceback.print_exc()
        print("Cleaning up...")
        #clear the VelocityUpdate event handler 
        ch.setOnVelocityUpdateHandler(None)  
        ch.close()
        return 1
    except EndProgramSignal as e:
        print(e)
        print("Cleaning up...")
        #clear the VelocityUpdate event handler 
        ch.setOnVelocityUpdateHandler(None)  
        ch.close()
        return 1
    finally:
        print("Press ENTER to end program.")
        readin = sys.stdin.readline()

main()

